"""
@file
@brief Shortcuts to graph_helper
"""

from .blockdiag_helper import draw_diagram
from .corrplot import Corrplot
from .graphviz_helper import run_dot
from .mapplot import plot_map_france, plot_map_france_polygon
