# Generated from \CSharpPreprocessorParser.g4 by ANTLR 4.10.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .CSharpPreprocessorParser import CSharpPreprocessorParser
else:
    from CSharpPreprocessorParser import CSharpPreprocessorParser
import java.util.Stack
import java.util.HashSet

# This class defines a complete listener for a parse tree produced by CSharpPreprocessorParser.


class CSharpPreprocessorParserListener(ParseTreeListener):

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorDeclaration.
    def enterPreprocessorDeclaration(self, ctx: CSharpPreprocessorParser.PreprocessorDeclarationContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorDeclaration.
    def exitPreprocessorDeclaration(self, ctx: CSharpPreprocessorParser.PreprocessorDeclarationContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorConditional.
    def enterPreprocessorConditional(self, ctx: CSharpPreprocessorParser.PreprocessorConditionalContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorConditional.
    def exitPreprocessorConditional(self, ctx: CSharpPreprocessorParser.PreprocessorConditionalContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorLine.
    def enterPreprocessorLine(self, ctx: CSharpPreprocessorParser.PreprocessorLineContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorLine.
    def exitPreprocessorLine(self, ctx: CSharpPreprocessorParser.PreprocessorLineContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorDiagnostic.
    def enterPreprocessorDiagnostic(self, ctx: CSharpPreprocessorParser.PreprocessorDiagnosticContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorDiagnostic.
    def exitPreprocessorDiagnostic(self, ctx: CSharpPreprocessorParser.PreprocessorDiagnosticContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorRegion.
    def enterPreprocessorRegion(self, ctx: CSharpPreprocessorParser.PreprocessorRegionContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorRegion.
    def exitPreprocessorRegion(self, ctx: CSharpPreprocessorParser.PreprocessorRegionContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessorPragma.
    def enterPreprocessorPragma(self, ctx: CSharpPreprocessorParser.PreprocessorPragmaContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessorPragma.
    def exitPreprocessorPragma(self, ctx: CSharpPreprocessorParser.PreprocessorPragmaContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#directive_new_line_or_sharp.
    def enterDirective_new_line_or_sharp(self, ctx: CSharpPreprocessorParser.Directive_new_line_or_sharpContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#directive_new_line_or_sharp.
    def exitDirective_new_line_or_sharp(self, ctx: CSharpPreprocessorParser.Directive_new_line_or_sharpContext):
        pass

    # Enter a parse tree produced by CSharpPreprocessorParser#preprocessor_expression.
    def enterPreprocessor_expression(self, ctx: CSharpPreprocessorParser.Preprocessor_expressionContext):
        pass

    # Exit a parse tree produced by CSharpPreprocessorParser#preprocessor_expression.
    def exitPreprocessor_expression(self, ctx: CSharpPreprocessorParser.Preprocessor_expressionContext):
        pass


del CSharpPreprocessorParser
